/* Copyright (c) 2008 PrimeBase Technologies GmbH, Germany
 *
 * PrimeBase Media Stream for MySQL
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Original author: Paul McCullagh
 * Continued development: Barry Leslie
 *
 * 2007-06-04
 *
 * H&G2JCtL
 *
 * Media Stream Tables.
 *
 */

#ifndef __OPENTABLE_MS_H__
#define __OPENTABLE_MS_H__

#include "CSDefs.h"
#include "CSFile.h"

#include "Engine_ms.h"

#include "Database_ms.h"

class MSOpenTablePool;

// Keep the value of MS_OT_BUFFER_SIZE less than 0XFFFF to avoid compiler warnings.
#ifdef DEBUG
#define MS_OT_BUFFER_SIZE		((csWord2)(512))
//#define MS_OT_BUFFER_SIZE		((csWord2)(0XFFF0))
#else
#define MS_OT_BUFFER_SIZE		((csWord2)(0XFFF0)) 
#endif

#define MS_UB_SAME_TAB			0
#define MS_UB_NEW_HANDLE		1
#define MS_UB_NEW_BLOB			2
#define MS_UB_RETAINED			3
#define MS_UB_NEW_RETAINED		4

/*
typedef struct MSUsedBlob {
	int					ub_state;
	csWord2				ub_col_index;
	csWord4				ub_repo_id;
	csWord8				ub_repo_offset;

	csWord2				ub_head_size;
	csWord8				ub_blob_size;
	csWord8				ub_blob_ref_id;
	csWord4				ub_blob_id;
	csWord4				ub_auth_code;
	char				ub_blob_url[PBMS_BLOB_URL_SIZE];
} MSUsedBlobRec, *MSUsedBlobPtr;
*/

class MSOpenTable : public CSRefObject, public CSPooled {
public:
	bool				inUse;
	bool				isNotATable;
	MSOpenTable			*nextTable;
	MSOpenTablePool		*myPool;

	CSFile				*myTableFile;

	MSRepository		*myWriteRepo;
	MSRepoFile			*myWriteRepoFile;

	MSTempLogFile		*myTempLogFile;

	char				myOTBuffer[MS_OT_BUFFER_SIZE];

	MSOpenTable();
	virtual ~MSOpenTable();

	virtual void returnToPool();

	void close();

	void createBlob(PBMSBlobURLPtr bh, csWord8 size, char *metadata, csWord2 metadata_size, CSInputStream *stream, CloudKeyPtr cloud_key = NULL, Md5Digest *checksum = NULL);
	void createBlob(PBMSBlobIDPtr blob_id, csWord8 blob_size, char *metadata, csWord2 metadata_size);
	void sendRepoBlob(csWord8 blob_id, csWord4 auth_code, bool info_only, CSHTTPOutputStream *stream);
	void useBlob(int type, csWord4 db_id, csWord4 tab_id, csWord8 blob_id, csWord4 auth_code, csWord2 col_index, csWord8 blob_size, csWord8 blob_ref_id, char *ret_blob_url);
//	void useBlob(int type, csWord4 db_id, csWord4 tab_id, csWord8 blob_id, csWord4 auth_code, csWord2 col_index, csWord8 blob_size, char **ret_blob_url);
//	void unUseBlobs();
//	void retainReferences();
	void releaseReference(csWord8 blob_id, csWord8 blob_ref_id);
	void freeReference(csWord8 blob_id, csWord8 blob_ref_id);
	void commitReference(csWord8 blob_id, csWord8 blob_ref_id);
	void checkBlob(CSStringBuffer *buffer, csWord8 blob_id, csWord4 auth_code, csWord4 temp_log_id, csWord4 temp_log_offset);
	bool deleteReferences(csWord4 temp_log_id, csWord4 temp_log_offset, bool *must_quit);

	void openForReading();
	void closeForWriting();

	csWord4 getTableID();
	MSTable *getDBTable();
	MSDatabase *getDB();

	void formatBlobURL(char *blob_url, csWord8 blob_id, csWord4 auth_code, csWord8 blob_size, csWord4 tab_id, csWord8 blob_ref_id);
	void formatBlobURL(char *blob_url, csWord8 blob_id, csWord4 auth_code, csWord8 blob_size, csWord8 blob_ref_id);
	void formatRepoURL(char *blob_url, csWord4 log_id, csWord8 log_offset, csWord4 auth_code, csWord8 blob_size);

	/* Make this object linkable: */
	virtual CSObject *getNextLink() { return iNextLink; }
	virtual CSObject *getPrevLink() { return iPrevLink; }
	virtual void setNextLink(CSObject *link) { iNextLink = link; }
	virtual void setPrevLink(CSObject *link) { iPrevLink = link; }

private:
	CSObject			*iNextLink;
	CSObject			*iPrevLink;

//	u_int				iUseSize;
//	u_int				iUseCount;
//	MSUsedBlobPtr		iUsedBlobs;
	

	void openForWriting();

public:
	static MSOpenTable *newOpenTable(MSOpenTablePool *pool);
};

class MSOpenTablePool : public CSRefObject, public CSPooled {
public:
	csWord4			myPoolTableID;						/* Non-zero if in the ID list. */
	bool			isRemovingTP;						/* Set to true if the table pool is being removed. */
	MSTable			*myPoolTable;
	MSDatabase		*myPoolDB;

	MSOpenTablePool();
	virtual ~MSOpenTablePool();

	MSOpenTable *getPoolTable();						/* Returns NULL if there is no table in the pool. */
	void returnOpenTable(MSOpenTable *otab);

	void addOpenTable(MSOpenTable *otab);
	void removeOpenTable(MSOpenTable *otab);

	void removeOpenTablesNotInUse();
	
	virtual void returnToPool();

#ifdef DEBUG
	void check();
#endif
	
	u_int getSize() { return iPoolTables.getSize(); }

private:
	MSOpenTable		*iTablePool;						/* A list of tables currently not in use. */
	CSLinkedList	iPoolTables;						/* A list of all tables in this pool */

public:
	static MSOpenTablePool *newPool(csWord4 db_id, csWord4 tab_id);
};

class MSTableList : public CSObject {
public:
	MSTableList();
	~MSTableList();

	static void startUp();
	static void shutDown();

	static void debug(MSOpenTable *otab);

	static MSOpenTable *getOpenTableByID(csWord4 db_id, csWord4 tab_id);
	static MSOpenTable *getOpenTableForDB(csWord4 db_id);
	static void releaseTable(MSOpenTable *otab);

	static bool removeTablePoolIfEmpty(MSOpenTablePool *pool);
	static void removeTablePool(MSOpenTablePool *pool);
	static void removeTablePool(MSOpenTable *otab);
	static void removeDatabaseTables(MSDatabase *database);

	static MSOpenTablePool *lockTablePoolForDeletion(csWord4 db_id, csWord4 tab_id, CSString *db_name, CSString *tab_name);
	static MSOpenTablePool *lockTablePoolForDeletion(MSTable *table);
	static MSOpenTablePool *lockTablePoolForDeletion(CSString *table_url);
	static MSOpenTablePool *lockTablePoolForDeletion(MSOpenTable *otab);
	static void unlockTablePool(MSOpenTablePool *pool);

private:
	static CSSyncOrderedList	*gPoolListByID;

};

#endif
